#pragma ident	"@(#)mount.c	1.11	97/02/10 SMI"

/*
 * Copyright (c) 1988 Sun Microsystems, Inc.
 */

#undef _KERNEL
#include <sys/types.h>
#include <locale.h>
#include <stdio.h>
#include <time.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/mnttab.h>
#include <sys/mount.h>
#include <fslib.h>

#define	MNTTYPE_EXT2FS	"ext2fs"

/*
 * The "hidden/nohidden" option is private. It is not approved for
 * use (see PSARC/1996/443), which is why it is not in the usage message.
 */
#define	RO		0
#define	RW		1

int roflag = 0;
static char *myopts[] = {
	"ro",
	"rw",
	NULL
};

main(int argc, char *argv[])
{
#ifndef MS_NOMNTTAB
	struct mnttab mnt;
#endif
	char	*mnt_special;
	char	*mnt_mountp;
	int c;
	char *myname;
	char typename[64];
	char buf[256];
	char *options, *value;
	extern int optind;
	extern char *optarg;
	int error = 0;
	int verbose = 0;
	int mflg = 0;
	int rflg = 0;
	int optcnt = 0;

	myname = strrchr(argv[0], '/');
	myname = myname ? myname + 1 : argv[0];
	(void) sprintf(typename, "%s_%s", MNTTYPE_EXT2FS, myname);
	argv[0] = typename;

	while ((c = getopt(argc, argv, "Vvmr?o:O")) != EOF) {
		switch (c) {
		case 'V':
		case 'v':
			verbose++;
			break;
		case '?':
			error++;
			break;
		case 'r':
			roflag++;
			break;
		case 'm':
#ifdef MS_NOMNTTAB
			mflg |= MS_NOMNTTAB;
#else
			rflg++;
#endif
			break;
		case 'o':
			options = optarg;
			while (*options != '\0') {
				switch (getsubopt(&options, myopts, &value)) {
				case RO:
					roflag++;
					break;
				case RW:
					break;
				default:
					(void) fprintf(stderr,
				    gettext("%s: illegal -o suboption -- %s\n"),
					    typename, value);
					error++;
					break;
				}
			}
			break;
		case 'O':
			mflg |= MS_OVERLAY;
			break;
		}
	}

	if (roflag == 0) {
		(void) fprintf(stderr, gettext("'ro' must be specified\n"));
		error++;
	}

	if (verbose && !error) {
		char *optptr;

		(void) fprintf(stderr, "%s", typename);
		for (optcnt = 1; optcnt < argc; optcnt++) {
			optptr = argv[optcnt];
			if (optptr)
				(void) fprintf(stderr, " %s", optptr);
		}
		(void) fprintf(stderr, "\n");
	}

	if (argc - optind != 2 || error) {
		/*
		 * don't hint at options yet (none are really supported)
		 */
		(void) fprintf(stderr, gettext(
		    "Usage: %s [generic options] [-o suboptions] "
		    "special mount_point\n"), typename);
		(void) fprintf(stderr, gettext(
		    "\tsuboptions are:\n"
		    "\t     ro,rw\n"));
		exit(32);
	}

	mnt_special = argv[optind++];
	mnt_mountp = argv[optind++];
#ifndef MS_NOMNTTAB
	mnt.mnt_special = mnt_special;
	mnt.mnt_mountp = mnt_mountp;
	mnt.mnt_fstype = MNTTYPE_EXT2FS;
	mnt.mnt_mntopts = buf;
#endif

	mflg |= MS_DATA;
	if (roflag) {
		mflg |= MS_RDONLY;
#ifndef MS_NOMNTTAB
		(void) strcpy(mnt.mnt_mntopts, "ro");
	} else {
		(void) strcpy(mnt.mnt_mntopts, "rw");
#endif
	}

	signal(SIGHUP,  SIG_IGN);
	signal(SIGQUIT, SIG_IGN);
	signal(SIGINT,  SIG_IGN);

	if (verbose) {
		(void) fprintf(stderr, "mount(%s, \"%s\", %d, %s",
		    mnt_special, mnt_mountp, mflg, MNTTYPE_EXT2FS);
	}
	if (mount(mnt_special, mnt_mountp, mflg, MNTTYPE_EXT2FS,
	    NULL, 0)) {
		if (errno == EBUSY) {
			fprintf(stderr, gettext(
			    "mount: %s is already mounted, %s is busy,\n\tor"
			    " allowable number of mount points exceeded\n"),
			    mnt_special, mnt_mountp);
		} else if (errno == EINVAL) {
			fprintf(stderr, gettext(
			    "mount: %s is not a Linux filesystem.\n"),
			    mnt_special);
		} else {
			perror("mount");
		}
		exit(32);
	}

#ifndef MS_NOMNTTAB
	if (!rflg && fsaddtomtab(&mnt))
		exit(1);
#endif
	return (0);
}
